/**
 * Implantation du serveur pour la salle de vente
* @author Pierre-Vincent ARNAUD, Lela TOURSI, Stphanie CADEAU, Anne VERNET, Audrey JACCARD
*/

import java.util.*;
import java.text.*;
import NommageFacile.*;
import SalleEnchere.*;
import SalleEnchere.SalleVentePackage.*;


/*
* Cette classe correspond au serveur de la salle de Vente
*/
public class Server {
    
    /* Cette mthode permet de lancer le serveur de la salle de vente
     * @param orb ORB du serveur d'vnements
     * @param args Contient le nom du fichier pour le catalogue, et la date de l'ouverture de la vente
     * @return Un Entier qui indique comment se termine la mthode
     * @throws UserException Problme avec l'ORB
     */
    static int run(org.omg.CORBA.ORB orb,String[] args) throws org.omg.CORBA.UserException {
	org.omg.PortableServer.POA rootPOA=org.omg.PortableServer.POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	org.omg.PortableServer.POAManager manager=rootPOA.the_POAManager();
	String nomObjet=new String("quad:10000:/sallesDeVentes/salle1");
	SalleVenteImpl s=new SalleVenteImpl(new SimpleDateFormat().parse(args[1],new ParsePosition(0)),args[0],orb,nomObjet);
	SalleVente sdv=s._this(orb);

	/* inscription au service de nommage */
	EasyNaming.inscriptionObjet(orb,nomObjet,sdv);
	manager.activate();
	/*Lancement de l'ORB */
	orb.run();
	return 0;
    }

    
    /** Main 
     * @param args <b>premier argument</b> le fichier contenant le catalogue des produits<br/>
     * <b>deuxime argument</b> date de l'ouverture de la salle de vente (ex. de format: "mm/dd/yy 2:02 PM" sur quad)
     */
    public static void main(String args[]) {

	/* On change l'ORB que possde Java par dfaut par ORBACUS*/
	java.util.Properties props=System.getProperties();
	props.put("org.omg.CORBA.ORBClass","com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass","com.ooc.CORBA.ORBSingleton");
	int status=0;
	org.omg.CORBA.ORB orb=null;

	/* Initialisation de l'ORB */
	try {
	    orb=org.omg.CORBA.ORB.init(args,props);
	    status=run(orb,args);
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	    status=1;
	}

	/* Destruction de l'ORB */
	if(orb!=null) {
	    try {
		orb.destroy();
	    }
	    catch(Exception ex) {
		ex.printStackTrace();
		status=1;
	    }
	}
	System.exit(status);
    }
}
