package easyNotify;

/**
 * classe permettant d'accder simplement  un service de notification
 *
 *
 *
 * @author Philippe.Lamarre@irin.univ-nantes.fr
 * $Id: EasyNotify.java,v 1.3 2001/12/14 14:37:12 lamarre Exp $
 **/

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosEventComm.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import java.util.*;


public abstract class EasyNotify {
    public static final int _viaORB = 1;
    public static final int _viaIOR = 2;
    public static final int _viaMachineName = 3;

    /**
     * ce peut tre soit via l'orb, soit via corbaloc, soit via l'ior
     **/
    protected int typeInit;

    protected String machineName;;
    protected String machinePort;

    protected String IOR;

    /**
     * l'ORB utilis
     **/
    protected ORB orb;

    /** 
     * la fabrique
     **/
    protected EventChannelFactory eventChannelFactory;

    /**
     * l'identit du cannal (format utile pour communication)
     **/
    protected IntHolder channelId;

    /**
     * le cannal 
     **/
    protected EventChannel eventChannel;


    /**
     *politiques choisies pour le canal d'vnement
     **/
    protected Property[] initialQoS;
    /**
     *politiques choisies pour le canal d'vnement
     **/
    protected Property[] initialAdmin;













    // *****************************************************************
    // *****************************************************************
    //
    //                         Les constructeurs
    //
    // *****************************************************************
    // *****************************************************************




    // *****************************************************************
    //                        cration du cannal
    // *****************************************************************

    /**
     * Initialisation via l'ORB et cration d'un cannal d'vnement
     * avec les politiques par dfaut
     **/
    public EasyNotify(ORB orb) 
	throws org.omg.CosNotification.UnsupportedQoS, org.omg.CosNotification.UnsupportedAdmin{
	this(orb, new Property[0], new Property[0]);
    }

  




    /**
     * initialisation via l'ORB et cration du cannal avec les
     * politiques fournies
     **/
    public EasyNotify(ORB orb, Property[] initialQoS, Property[] initialAdmin) 
	throws org.omg.CosNotification.UnsupportedQoS,  org.omg.CosNotification.UnsupportedAdmin {
	// obtention du channelFactory
	obtainEventChannelFactoryViaORB(orb);

	// cration du cannal d'vnements
	createChannel(initialQoS, initialAdmin);
    }





    /**
     * initialisation via corbaloc et cration d'un cannal avec les
     * proprits par dfaut
     **/
    public EasyNotify(org.omg.CORBA.ORB orb, String notificationServiceMachine, String notificationServicePort) 
	throws org.omg.CORBA.SystemException, org.omg.CosNotification.UnsupportedQoS, org.omg.CosNotification.UnsupportedAdmin {
	
	obtainEventChannelFactoryViaCorbaloc(orb, notificationServiceMachine, notificationServicePort);

	if (eventChannelFactory == null) {
	    System.err.println("Returned IOR is not a NotificationContext");
	} else {
	    initialQoS = new Property[0];
	    initialAdmin = new Property[0];
	    createChannel(initialQoS, initialAdmin);
	}
    }




    // *****************************************************************
    //                   Utilisation cannal existant
    // *****************************************************************

    /**
     * Initialisation via l'ORB et rcupration d'un cannal
     * d'vnements existant
     **/
     public EasyNotify(ORB orb, int channelId) throws ChannelNotFound {
	// obtention du channelFactory
	obtainEventChannelFactoryViaORB(orb);

	// obtention du cannal
	getChannel(channelId);
    }



    /**
     * initialisation via corbaloc et obtention d'un cannal existant
     **/
    public EasyNotify(org.omg.CORBA.ORB orb, String notificationServiceMachine, String notificationServicePort, int channelId) 
	throws org.omg.CORBA.SystemException, org.omg.CosNotifyChannelAdmin.ChannelNotFound
	 {
	
	obtainEventChannelFactoryViaCorbaloc(orb, notificationServiceMachine, notificationServicePort);

	if (eventChannelFactory == null) {
	    System.err.println("Returned IOR is not a NotificationContext");
	} else {
	    getChannel(channelId);
	}
    }










    // *****************************************************************
    //              utilitaires pour les constructeurs
    // *****************************************************************


    /**
     * cration du cannal d'vnements
     **/
    private void createChannel(Property[] initialQoS, Property[] initialAdmin) 
    throws org.omg.CosNotification.UnsupportedQoS,  org.omg.CosNotification.UnsupportedAdmin{
	// mmorisation des politiques choisies
	System.out.println("Creation du cannal");
	this.initialQoS = initialQoS;
	this.initialAdmin = initialAdmin;

	// cration du holder 
	channelId = new IntHolder();

	// obtention du cannal
	eventChannel = eventChannelFactory.create_channel(initialQoS, initialAdmin, channelId);

    }




    /**
     * rcupration du cannal
     **/
    private void getChannel(int channelId) throws ChannelNotFound {
	System.out.println("Rcupration du canal n"+channelId); 
	// mmorisation de l'identit du cannal
	this.channelId = new IntHolder(channelId);

	// rcupration du cannal
	eventChannel = eventChannelFactory.get_event_channel(channelId);

    }








    /**
     * tout ce qu'il faut faire pour obtenir une fabrique via l'orb
     **/
    private void obtainEventChannelFactoryViaORB(ORB orb) {
	// mmorisation du type d'initialisation
	this.typeInit = _viaORB;

	// mmorisation de l'ORB
	this.orb = orb;

	// rsolution
	try {
	    org.omg.CORBA.Object obj = orb.resolve_initial_references("NotificationService");
	    eventChannelFactory = EventChannelFactoryHelper.narrow(obj);
	} catch (org.omg.CORBA.ORBPackage.InvalidName e) {
	    System.err.println("Nom invalide : NotificationService");
	}
    }






    /**
     * tout ce qu'il faut faire pour obtenir la fabrique via un nom de
     * machine et un numro de port
     **/
    private void obtainEventChannelFactoryViaCorbaloc(org.omg.CORBA.ORB orb, String notificationServiceMachine, String notificationServicePort) throws org.omg.CORBA.SystemException {
	// mmorisation du type d'initialisation
	this.typeInit = _viaMachineName;

	// mmorisation de l'orb
	this.orb = orb;


	// mmorisation des coordonnes
	this.machineName = notificationServiceMachine;
	this.machinePort = notificationServicePort;

	// construcion de la chane permettant d'obtenir un corbaloc
	String chaine = "corbaloc:iiop:"+machineName+":"+machinePort+"/DefaultEventChannelFactory";
	System.out.println("Rcupration du channel Factory : " + chaine);
	org.omg.CORBA.Object obj = (org.omg.CORBA.Object)orb.string_to_object(chaine);
	eventChannelFactory = EventChannelFactoryHelper.narrow(obj);
    }

    









    /**
     * type d'initialisation utilise
     **/

    public int initType() {
	return typeInit;
    }




    /**
     * Identit du cannal
     **/

    public IntHolder getChannelId() {
	return channelId;
    }





    

    // *****************************************************************
    // *****************************************************************
    //
    //               Les mthodes abstraites  implmenter
    //
    // *****************************************************************
    // *****************************************************************

    /**
     * comment se connecter (en particuler en tant que supplier ou
     * consumer)
     **/
    public abstract void connect() throws org.omg.CosNotifyChannelAdmin.AdminLimitExceeded, org.omg.CosEventChannelAdmin.AlreadyConnected;


    public abstract void disconnect();

}
