package easyNotify;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CORBA.*;
import java.util.*;

public class EasyStructuredEvent {

    protected StructuredEvent structuredEvent;
    protected String domainName;
    protected String typeName;
    protected String eventName;
    protected Property[] body;
    protected Property[] variableHeader;
    protected EventHeader eventHeader;
    protected ORB orb;
    /**
     * construction de l'vnement dans sa totalit
     **/
    public EasyStructuredEvent(ORB orb,
			       String domainName, 
			       String typeName,
			       String eventName) {
	this.orb = orb;
	eventHeader = createHeader(domainName, typeName, eventName);
	setBody(createBody());
    }


    /**
     * lorsque le body doit tre spcifi, on peut soit 
     **/
    public EasyStructuredEvent(ORB orb,
			       String domainName, 
			       String typeName,
			       String eventName,
			       Property[] body) {
	this.orb = orb;
	eventHeader = createHeader(domainName, typeName, eventName);
	setBody(body);
    }


    public EasyStructuredEvent(StructuredEvent event) {
	structuredEvent = event;
	eventHeader = event.header;
	domainName = event.header.fixed_header.event_type.domain_name;
	typeName = event.header.fixed_header.event_type.type_name;
	eventName = event.header.fixed_header.event_name;
	body = event.filterable_data;
    }


    /**
     * positionnement du body.
     *
     * attention ne peut tre fait que pour les cas o l'on a cr un
     * vnement nous mme pour l'envoy. Dans le cas contraire, ce
     * n'est pas modifiable, l'opration est donc sans effet.
     **/
    protected void setBody(Property[] body) {
	if (orb != null) {
	    this.body = body;
	    structuredEvent = new StructuredEvent(eventHeader, body, orb.create_any());
	}
    }



    /**
     * cration du header
     *
     * @param domainName nom du domaine industriel pour lequel
     * l'vnement est produit
     * 
     * @param typeName nom qui catgorise de manire unique
     * l'vnement dans le domaine industriel envisag.
     *
     * @param eventName identification de l'instance de l'vnement.
     **/
    public EventHeader createHeader(String domainName, 
				    String typeName,
				    String eventName) {
	this.domainName = domainName;
	this.typeName = typeName;
	this.eventName = eventName;

	//                                  partie fixe du header

	// cration du type 
	EventType eventType = new EventType(domainName, typeName);

	// cration du nom de l'vnement
	FixedEventHeader fixedEventHeader = new FixedEventHeader(eventType, eventName);

	
	//                                  partie variable
	variableHeader = createHeaderProperties();



	// cration du header 
	return  new EventHeader(fixedEventHeader, variableHeader);
    }




    /**
     * partie variable du header
     *
     * si l'on veut utiliser le timeout ou autres choses trs
     * intressantes, c'est ici que cela se passe
     **/
    public Property[] createHeaderProperties() {
	return new Property[0];
    }

    /**
     * c'est ici que l'on remplie le corps de l'vnement (les valeurs
     * qu'il transporte) sous forme de proprits
     *
     * on peut surcharger cette mthode si le calcul peut tre ralis
     * en local (ou utiliser un passage en paramtre lors de la
     * cration).
     **/
    public  Property[] createBody() {
	return new Property[0];
    }

    
    public StructuredEvent toStructuredEvent() {
	return structuredEvent;
    }


    public Property[] getBody(){
	return body;
    }
    
    public String getDomainName() {
	return domainName;
    }

    public String getTypeName() {
	return typeName;
    }
    
    public String getEventName() {
	return eventName;
    }

    public Any string2Any(String param) {
	Any toReturn = orb.create_any();
	toReturn.insert_string(param);
	return toReturn;
    }

    public Any long2Any(int param) {
	Any toReturn = orb.create_any();
	toReturn.insert_long(param);
	return toReturn;
    }

    String propertyToString(Property[] properties) {
	String toReturn = "<Property[]>";
	if (properties != null) {
	    for (int i = 0 ; i < properties.length ; i++) {
		// il faudrait extraire du type any... mais on ne sait pas
		// troujours le faire...  tudier.
		toReturn = toReturn + "<"+properties[i].name+">"+"NOT_IMPLEMETED"+"</"+properties[i].name+">";
	    } 
	} else {
	    toReturn = toReturn + "null";
	}
	return toReturn + "</Property[]>";
    }

    public String fixedHeaderToString() {
	return "<FixedHeader><DomainName>"+domainName+"</DomainName><TypeName>"+typeName+"</TypeName><EventName>"+eventName+"</EventName><FixedHeader>";
    }
    public String variableHeaderToString() {
	return "<VariableHeader>"+propertyToString(variableHeader)+"</VariableHeader>";
    }

    public String filterableBodyToString() {
	return "<FilterableBody>"+propertyToString(body)+"</FilterableBody>";
    }
   
    public String remainingBodyToString() {
	return "<RemainingBody>NOT_IMPLEMENTED</RemainingBody>";
    }
   
    public String eventHeaderToString() {
	return "<EventHeader>"+fixedHeaderToString()+variableHeaderToString()+"</EventHeader>";
    }

    public String eventBodyToString() {
	return "<EventBody>"+filterableBodyToString()+remainingBodyToString()+"</EventBody>";
    }

    public String toString() {
	return "<StructuredEvent>"+eventHeaderToString()+eventBodyToString()+"</StructuredEvent>";
    }

}

