import SalleEnchere.SalleVente;
import SalleEnchere.SalleVentePackage.*;
  
import VenteAuxEncheres.Catalogue;
import VenteAuxEncheres.Produit;
import VenteAuxEncheres.Enchere;
import VenteAuxEncheres.Personne;
import VenteAuxEncheres.Statut;
import VenteAuxEncheres.EventServiceIdHolder;

import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;

/**
 * Cette classe reprsente un client d'une salle des ventes.
 * Les interactions avec la salle des ventes sont ralises via une interface
 * graphique. 
 * 
 * @author Kvin Simonin
 * @author Nicolas Pezron
 * @author <a href="mailto:anthony.milan@voila.fr">Anthony Milan</a>
 * @author <a href="mailto:yves.castelain@sncf.fr">Yves Castelain</a>
 */
public class Client extends ClientUI implements easyNotify.EasyStructuredEventDealer {
    
    /** L'enchre courante. */
    private Enchere enchereCourante = null; 
    
    /** L'identifiant donn par la salle des ventes */
    private int monId; 
    
    /** Le nom du client. */
    protected String nom = ""; 
    
    /** Le prnom du client. */
    protected String prenom = "";
    
    /** Le solde du client. */
    private int solde;
    
    /** La somme dpense depuis le dbut de la participation  la vente.*/
    private int depense = 0;
    
    /** Le catalogue de la salle des ventes. */
    private Catalogue catalogue;
    
    /** La salle des ventes. */
    private SalleVente salleDesVentes;
    
    /** Encapsulation des mcanismes CORBA. */
    private ClientCorba corba;

    /**
     * Construit un client avec un solde de 50000 euros.
     *
     */
    public Client() {
	this(50000); 
    }
    
    /**
     * Construit un client.
     *
     * @param solde un solde particulier
     */
    public Client(int solde) {
	/*this.nom = nom;
	this.prenom = prenom;*/
	setSolde(solde);
	// cration du thread corba et dmarrage
	corba = new ClientCorba(new String[0]);
	corba.start(); 
    }
    
    /**
     * The main method.
     *
     * @param args arguments de la ligne de commande
     */
    public static void main(String[] args) {	
	String nom;
	String prenom;
	int solde = 0;
	// cration de l'IHM 
	try {
	    Client client = new Client();
	    if (args.length == 1) {
		client.setSolde(Integer.parseInt(args[0]));
	    }
	    client.show();
	} catch(NumberFormatException ex) {
	    System.err.println("usage : java Client [solde]");
	    System.exit(1);
	}
    }
    
    /**
     * Cette mthode traite les vnements mis par la salle des ventes.
     *
     * @param event un vnement de la salle des ventes transmis par le service
     *              de notification
     */
    public synchronized void treatEvent(org.omg.CosNotification.StructuredEvent ev) {
	SdvEvent event = null;
	try {
	    event = new SdvEvent(ev);
	} catch(easyNotify.BadEvent e) {
	    System.err.println("Un vnement sur le service de notification non pertinent pour l'application");
	    return ;
	}
	
	// Fermeture de la salle des ventes ...
	if (event.getMessage().equals("FERMETURE")) {
	    JOptionPane.showMessageDialog(this,
					  "La salle des ventes est ferme",
					  "",
					  JOptionPane.WARNING_MESSAGE);
	    menuRetirer.setEnabled(false);
	    menuParticiper.setEnabled(false);
	    menuCatalogue.setEnabled(false);	    
	    menuConnect.setEnabled(true);
	    corba.endNotificationService();
	    //salleDesVentes=null;
	    return;
	}
	// Autres cas, on rcupre l'enchre courante
	enchereCourante = event.getEnchere();
	
	// Selon le staut de l'enchre, le traitement adquat est excut
	switch(enchereCourante.leStatut.value()) {
	case Statut._encheri :
	case Statut._enBaisse :
	    fireEnchereHasChanged();
	    break;
	    // ici une seule info  mettre  jour
	case Statut._uneFois :
	case Statut._deuxFois :
	case Statut._troisFois :
	    setStatutCourant(printStatut(enchereCourante.leStatut));
	    break;
	case Statut._adjuge :
	    // modif de l'affichage, de l'historique
	    fireEnchereHasChanged();
	    addHisto(enchereCourante);
	    // ventuellement de la somme dpense et du solde
	    // si le client obtient l'enchre
	    if (enchereCourante.idPersonne == monId) {
		setSolde(solde - enchereCourante.prixCourant);
		setDepense(depense + enchereCourante.prixCourant);
	    }
	}
    }
    
    /**
     * Affecte le solde.
     *
     * @param solde le solde du client
     */
    protected void setSolde(int solde) {
	// pour l'affichage en MVC faire un listener et envoyer un vnement :(
	super.setSolde(solde); 
	this.solde = solde;
    }
    
    /**
     * Affecte la dpense.
     *
     * @param depense somme dpense par le client
     */
    protected void setDepense(int depense) {
	super.setDepense(depense);
	this.depense = depense;
    }
    
    /**
     * Cette mthode permet  l'utilisateur d'enchrir.
     *
     * @param evt
     */
    protected void onEncherir(ActionEvent evt) {	
	// il faut rcuprer la valeur de l'enchre et enchrir  la sdv
	// PEUT ETRE VERIFIER QU'iL RESTE DE L'ARGENT !!!
	try {
	    int newEnchere = Integer.parseInt(textFieldEnchere.getText());
	    if (newEnchere <= solde) {
		try {
		    salleDesVentes.encherir(monId,
					    enchereCourante.idProduitCourant,
					    newEnchere);
		} catch(EnchereBasse ex) {
		    JOptionPane.showMessageDialog(this,
						  "L'enchre effectue est trop basse",
						  "",
						  JOptionPane.WARNING_MESSAGE);
		} catch(MauvaisProduit ex) {
		    JOptionPane.showMessageDialog(this,
						  "Vous n'avez pas effectu l'enchre sur le produit courant : "+ex.getMessage(),
						  "",
						  JOptionPane.WARNING_MESSAGE);
		}
	    } else {
		JOptionPane.showMessageDialog(this,
					      "Vous n'avez pas assez d'argent pour faire une telle enchre",
					      "",
					      JOptionPane.WARNING_MESSAGE);
	    }
	} catch(NumberFormatException ex){
	    JOptionPane.showMessageDialog(this,
					  "Vrifier la syntaxe de votre enchre",
					  "Format erron",
					  JOptionPane.ERROR_MESSAGE);
	}
    }
    
    /**
     * Cette mthode permet de se connecter  la salle des ventes.
     *
     * @param evt
     */
    protected void onConnect(ActionEvent evt) {
	// rcupration de la salle des ventes
	salleDesVentes = corba.getSalleVente();
	
	if (salleDesVentes != null) {	
	    menuParticiper.setEnabled(true);
	    menuCatalogue.setEnabled(true);
	    menuConnect.setEnabled(false);
	} else {
	    JOptionPane.showMessageDialog(this,
					  "La connexion a chou",
					  "",
					  JOptionPane.WARNING_MESSAGE);
	}
    }
    
    /**
     * Permet  l'utilisateur d'obtenir le catalogue.
     *
     * @param evt
     */
    protected void onCatalogue(ActionEvent evt) {
	System.out.println(salleDesVentes);
	catalogue = salleDesVentes.getCatalogue();
	menuCatalogue.setEnabled(false);
	showCatalogue();
    }
    
    /**
     * Permet  l'utilisateur de participer  une vente.
     * Rcupre le catalogue s'il ne l'a pas.
     *
     * @param evt
     */  
    protected void onParticiper(ActionEvent evt) {
	try {
	    PersonneUI p = new PersonneUI(this);
	    p.show();
	    if (nom == "" || prenom == "") {
		JOptionPane.showMessageDialog(this,
					      "Participation annule",
					      "",
					      JOptionPane.WARNING_MESSAGE);
		return;
	    }

	    org.omg.CORBA.IntHolder idPersonne = new org.omg.CORBA.IntHolder();
	    EventServiceIdHolder ev = new EventServiceIdHolder();
	    // accs  la salle des ventes
	    
	    enchereCourante = salleDesVentes.entree(new Personne(nom, prenom), idPersonne, ev); 
	    if (catalogue == null)
		onCatalogue(null);
	    // identifiant donn par la sdv
	    monId = idPersonne.value;
	    
	    // Dmarrage du service de notification
	    corba.startNotificationService(this, ev);
	    
	    // mise  jour UI
	    fireEnchereHasChanged();
	    boutonEncherir.setEnabled(true);
	    menuParticiper.setEnabled(false);
	    menuRetirer.setEnabled(true);
	} catch(SalleFermee ex) {
	    JOptionPane.showMessageDialog(this,
					  "La salle des ventes est ferme",
					  "",
					  JOptionPane.WARNING_MESSAGE);
	} catch(org.omg.CORBA.UNKNOWN ex) {
	    System.err.println(ex.getMessage());
	}
    }
    
    /**
     * L'acheteur se retire de la salle des ventes.
     */
    protected void onRetirer() {
	// quitte la salle des ventes
	salleDesVentes.sortie(monId);
	corba.endNotificationService();
	menuRetirer.setEnabled(false);
	menuCatalogue.setEnabled(false);
	menuParticiper.setEnabled(true);
	boutonEncherir.setEnabled(false);
    }
    
    /**
     * Quitter l'application
     */
    protected void onQuitter() {
	try {
	// on pense  prvenir la salle des ventes
	    if (salleDesVentes != null) {
		salleDesVentes.sortie(monId);
		corba.endNotificationService();
	    }
	}catch(Exception ex) {}
	
	finally {
	    System.exit(0);
	}
    }
    
    /**
     * Affiche le catalogue sur l'interface.
     */
    private void showCatalogue() {
	String[] header = {"Id", "Nom", "Description", "Prix Dpart", "Vendu", "Prix Vente", "Acqureur"};
	javax.swing.table.DefaultTableModel dataModel = new javax.swing.table.DefaultTableModel( header, 0 );
	
	for(int i=0; i< catalogue.produits.length; i++) {
	    Object[] row = new Object[dataModel.getColumnCount()];
	    row[0] = new Integer(catalogue.produits[i].id);
	    row[1] = catalogue.produits[i].nom;
	    row[2] = catalogue.produits[i].description;
	    row[3] = new Integer(catalogue.produits[i].prixDepart);
	    row[4] = catalogue.produits[i].vendu ? "oui" : "non";
	    row[5] = catalogue.produits[i].vendu ? ""+catalogue.produits[i].prixVente : "";
	    row[6] = "";
	    dataModel.addRow(row);
	}
	tableCatalogue.setModel(dataModel);
    }
    
    /**
     * Rpertorie une enchre dans l'historique.
     *
     * @param e une enchre attribue
     */
    private void addHisto(Enchere e) {
	javax.swing.table.DefaultTableModel model = (javax.swing.table.DefaultTableModel) tableCatalogue.getModel(); //tableHistorique.getModel();
	tableCatalogue.setValueAt(new Integer(e.idPersonne),e.idProduitCourant, 6);
	if (e.leStatut.value() == Statut._adjuge) {
	    model.setValueAt("oui", e.idProduitCourant, 4);
	    model.setValueAt(new Integer(e.prixCourant), e.idProduitCourant, 5);
	}
	model.fireTableDataChanged();
	tableCatalogue.updateUI();
    }
    
    /**
     * Cette mthode est dclanche quand une nouvelle enchre est faite.
     * Mise  jour de l'interface.
     */
    private void fireEnchereHasChanged() {
	if (enchereCourante == null) 
	    return;
	setArticleCourant(catalogue.produits[enchereCourante.idProduitCourant].nom+" : "+catalogue.produits[enchereCourante.idProduitCourant].description);
	setPrixCourant(enchereCourante.prixCourant);
	setEncherisseurCourant(""+enchereCourante.idPersonne) ;
	setStatutCourant(printStatut(enchereCourante.leStatut));
	panelArticle.updateUI();
    }
    
    /**
     * Donne une reprsentation sous forme de chaine d'un statut.
     *
     * @param stat un statut d'enchre
     * @return une chane dcrivant le statut
     */
    private String printStatut(Statut stat) {
	String res="";
	switch(stat.value()) {
	case Statut._encheri :
	    res = "Encheri";
	    break;
	case Statut._enBaisse :
	    res = "En baisse";
	    break;
	case Statut._uneFois :
	    res = "Une fois";
	    break;
	case Statut._deuxFois :
	    res = "Deux fois";
	    break;
	case Statut._troisFois :
	    res = "Trois fois";
	    break;
	case Statut._adjuge :
	    res = "Adjuge";
	}
	return res;
    }
}
