import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.PortableServer.*;

import SalleEnchere.*;

/**
 * Cette classe encapsule les mcanismes CORBA du client.
 * Elle est charge des connexions au service de nommage et permet de dmarrer
 * le service de notifications.
 *
 * @author Kvin Simonin
 * @author Nicolas Pezron
 * @author <a href="mailto:anthony.milan@voila.fr">Anthony Milan</a>
 * @author <a href="mailto:yves.castelain@sncf.fr">Yves Castelain</a>
 */
public class ClientCorba extends Thread {
        
    /** Les arguments de la ligne de commandes. */
    private String[] arguments;
    
    /** Le POA pour le service de notification. */
    private static POA rootPOA;
    
    /** L'ORB utilis par le client. */
    private static org.omg.CORBA.ORB orb;

    /** . */
    private static EventListenerSDV notifyListener; 

    /**
     * Crer un client corba.
     */
    public ClientCorba() {
    }

    /**
     * Crer un client corba.
     *
     * @param args arguments de la ligne de commande
     */
    public ClientCorba(String[] args) {
	arguments = args;
    }

    /** Mthode excute au dmarrage du <code>thread</code>. */
    public void run() {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass", "com.ooc.CORBA.ORBSingleton");
	System.setProperties(props);
	
	orb = org.omg.CORBA.ORB.init(arguments, props);
	run(orb);
    }

    /**
     * Mthode run de l'ORB.
     *
     * @param orb
     */
    private void run(ORB orb) {	
	try {
	    // rcupration du POA car nous sommes serveur du service
	    // de notification
	    org.omg.CORBA.Object obj = orb.resolve_initial_references("RootPOA");
	    rootPOA = POAHelper.narrow(obj);

	    POAManager manager = rootPOA.the_POAManager();
	    manager.activate();

	    orb.run();	    
	} catch (org.omg.CORBA.ORBPackage.InvalidName  e) {
	    System.err.println("ClientCorba.run(ORB orb)");
	    e.printStackTrace();
	} catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive e) {
	    System.err.println("ClientCorba.run(ORB orb)");
	    e.printStackTrace();
	}
    }

    /**
     * Dmarre "l'coute" sur le service de notification.
     *
     * @param o
     * @param ev 
     */
    public void startNotificationService(easyNotify.EasyStructuredEventDealer easy,
						VenteAuxEncheres.EventServiceIdHolder ev) {
	notifyListener = new EventListenerSDV(orb,
							       rootPOA,
							       easy,
							       ev);	
	notifyListener.connect();
    }
    
    public void endNotificationService() {
	notifyListener.disconnect();
    }
    
    /**
     * Permet d'obtenir une salle des ventes.
     * Cette mthode utilise le service de nommage.
     *
     * @return une salle des ventes
     */
    public SalleVente getSalleVente() {    
	String nomObjet = new String("quad:10000:/sallesDeVentes/salle1");
	org.omg.CORBA.Object obj = 
	    // utilisation du paquetage d'un autre groupe du DESS
	    NommageFacile.EasyNaming.resoudreReferenceObjet(orb,
							    nomObjet);
	// Transtypage
	return (SalleVente) SalleVenteHelper.narrow(obj);
    }
}


