import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotification.*;

import easyNotify.*;   // paquetage de Philippe Lamarre

/**
 * <p>Cette classe permet d'couter les vnements du service de notification.
 * Elle encapsule les mthodes lies  CORBA pour un autre 
 * EasyStructuredEventDealer <code>eventDealer</code>.</p>
 * <p>Le traitement de l'vment est dlgu  <code>eventDealer</code>.
 * La diffusion des vnements est effectue en mode <i>push</i>.
 * </p>
 *
 * @author Kvin Simonin
 * @author Nicolas Pezron
 * @author <a href="mailto:anthony.milan@voila.fr">Anthony Milan</a>
 * @author <a href="mailto:yves.castelain@sncf.fr">Yves Castelain</a>
 */
public class EventListenerSDV implements EasyStructuredEventDealer {

    /** L'eventDealer auquel le traitement des vnements est dlgu. */
    protected EasyStructuredEventDealer eventDealer;

    /** Permet la connexion au service de notification. */
    protected EasyNotifyStructuredPushConsumer easyConsumer;

    /**
     * Cration d'un <code>EventListenerSDV</code>.
     *
     * @param orb l'orb utilis
     * @param poa
     * @param eventDealer le traitement des vnements est dlgu  cet objet
     * @param ev information sur la machine o s'excute le service de
     *           notification
     */
    public EventListenerSDV(ORB orb,
			    POA poa,
			    EasyStructuredEventDealer eventDealer,
			    VenteAuxEncheres.EventServiceIdHolder ev) {
	VenteAuxEncheres.EventServiceId service = ev.value;
	this.eventDealer = eventDealer;
	try {
	    // indique la machine o s'excute le service de notifications
	    easyConsumer = 
		new EasyNotifyStructuredPushConsumer(orb, poa,
						     service.nomMachine,
						     ""+service.numPort,
						     service.numCanal,
						     this);
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
    
    /**
     * Connection au service de notifications.
     * En attente d'vnements.
     */
    public void connect() {
	try {
	    easyConsumer.connect();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
    /**
     * Dconnection au service de notifications.
     */
    public void disconnect() {
	try {
	    easyConsumer.disconnect();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
    /**
     * Cette mthode traite les vnments du service de notification.
     *
     * @param event un vnement du service de notification
     */
    public void treatEvent(StructuredEvent event) {
	eventDealer.treatEvent(event);
    }
}
